@@ -6,7 +6,6 @@ module GUI.GtkExtras where
 
 import Graphics.UI.GtkInternals
 import Graphics.UI.Gtk (Rectangle)
-import System.Glib.GError
 import System.Glib.MainLoop
 import Graphics.Rendering.Pango.Types
 import Graphics.Rendering.Pango.BasicTypes
@@ -14,9 +13,15 @@ import Graphics.UI.Gtk.General.Enums (StateType, ShadowType)
 
 import Foreign
 import Foreign.C
-import Control.Monad
 import Control.Concurrent.MVar
 
+#if !(mingw32_HOST_OS || mingw32_TARGET_OS)
+import System.Glib.GError
+import Control.Monad
+#endif
+
+#include "windows_cconv.h"
+
 waitGUI :: IO ()
 waitGUI = do
   resultVar <- newEmptyMVar
@@ -91,7 +96,7 @@ launchProgramForURI uri = do
             1       -- SW_SHOWNORMAL
     return True
 
-foreign import stdcall unsafe "shlobj.h ShellExecuteA"
+foreign import WINDOWS_CCONV unsafe "shlobj.h ShellExecuteA"
     c_ShellExecuteA :: Ptr ()  -- HWND hwnd
                     -> CString -- LPCTSTR lpOperation
                     -> CString -- LPCTSTR lpFile
